@@ -15,10 +15,8 @@ import six
 from saml2.samlp import Extensions
 from saml2 import xmldsig as ds
 
-from future.backports.urllib.parse import parse_qs
-from future.backports.urllib.parse import urlparse
-
 from six import StringIO
+from six.moves.urllib import parse
 
 from paste.httpexceptions import HTTPSeeOther, HTTPRedirection
 from paste.httpexceptions import HTTPNotImplemented
@@ -104,7 +102,7 @@ class SAML2Plugin(object):
         self.discosrv = discovery
         self.idp_query_param = idp_query_param
         self.logout_endpoints = [
-            urlparse(ep)[2] for ep in config.endpoint("single_logout_service")
+            parse.urlparse(ep)[2] for ep in config.endpoint("single_logout_service")
         ]
         try:
             self.metadata = self.conf.metadata
@@ -158,7 +156,7 @@ class SAML2Plugin(object):
             environ["wsgi.input"] = StringIO(body)  # restore the request body
             # as a stream so that everything seems untouched
 
-        post = parse_qs(body)  # parse the POST fields into a dict
+        post = parse.parse_qs(body)  # parse the POST fields into a dict
 
         logger.debug("identify post: %s", post)
 
@@ -218,7 +216,7 @@ class SAML2Plugin(object):
             query = environ.get(key)
             if query:
                 try:
-                    _idp_entity_id = dict(parse_qs(query))[self.idp_query_param][0]
+                    _idp_entity_id = dict(parse.parse_qs(query))[self.idp_query_param][0]
                     if _idp_entity_id in idps:
                         idp_entity_id = _idp_entity_id
                     break
@@ -239,7 +237,7 @@ class SAML2Plugin(object):
                 if self.wayf:
                     if query:
                         try:
-                            wayf_selected = dict(parse_qs(query))["wayf_selected"][0]
+                            wayf_selected = dict(parse.parse_qs(query))["wayf_selected"][0]
                         except KeyError:
                             return self._wayf_redirect(came_from)
                         idp_entity_id = wayf_selected
@@ -273,7 +271,6 @@ class SAML2Plugin(object):
     #### IChallenger ####
     # noinspection PyUnusedLocal
     def challenge(self, environ, _status, _app_headers, _forget_headers):
-
         _cli = self.saml_client
 
         if "REMOTE_USER" in environ:
@@ -348,7 +345,7 @@ class SAML2Plugin(object):
                     )
 
                 if _cli.authn_requests_signed:
-                    _sid = saml2.s_utils.sid()
+                    _sid = sid()
                     req_id, msg_str = _cli.create_authn_request(
                         dest,
                         vorg=vorg_name,
@@ -359,7 +356,10 @@ class SAML2Plugin(object):
                     _sid = req_id
                 else:
                     req_id, req = _cli.create_authn_request(
-                        dest, vorg=vorg_name, sign=False, extensions=extensions
+                        dest,
+                        vorg=vorg_name,
+                        sign=False,
+                        extensions=extensions,
                     )
                     msg_str = "%s" % req
                     _sid = req_id
@@ -385,7 +385,7 @@ class SAML2Plugin(object):
                 if (environ["PATH_INFO"]) in ret and ret.split(environ["PATH_INFO"])[
                     1
                 ] == "":
-                    query = parse_qs(environ["QUERY_STRING"])
+                    query = parse.parse_qs(environ["QUERY_STRING"])
                     sid = query["sid"][0]
                     came_from = self.outstanding_queries[sid]
             except:
@@ -472,7 +472,7 @@ class SAML2Plugin(object):
 
         is_request = "SAMLRequest" in query
         is_response = "SAMLResponse" in query
-        has_content_length = "CONTENT_LENGTH" in environ or environ["CONTENT_LENGTH"]
+        has_content_length = environ.get("CONTENT_LENGTH")
 
         if not has_content_length and not is_request and not is_response:
             logger.debug("[identify] get or empty post")
@@ -672,7 +672,7 @@ def make_plugin(
     discovery="",
     idp_query_param="",
 ):
-    if saml_conf is "":
+    if saml_conf == "":
         raise ValueError("must include saml_conf in configuration")
 
     if remember_name is None:
